
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#ifndef NO_DEBUG
#include <stream.h>
#define VERSION "DEBUG VERSION"
#define DEBUG
#else
#define VERSION "V1.0.2"
#endif

#include <qdir.h>
#include <qaccel.h>
#include "zauralign.h"

#include "zauralignview.h"
#include "zauraligndoc.h"

#undef ONLINE_HELP

#ifdef ZLOG
QTextStream zlog;
#endif

ZauralignApp::ZauralignApp(QPEApplication *app) : QMainWindow(0)
{
  #ifdef DEBUG  
  cout << "Zauralign="<<VERSION<<endl;
  #endif

  setCaption(tr("Zauralign " VERSION));
  mApp=app;

  mConfig=new KConfig("zauralign",KConfig::User);

  #ifdef ZLOG
  QString file=QString("%1zauralign.log").arg(app->qpeDir());
  mFile.setName(file);
  if ( mFile.open(IO_WriteOnly) )
  { 
    zlog.setDevice(&mFile);
    zlog << "Opened log file " VERSION << endl;
  }
  #endif


  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initActions();
  initMenuBar();

  // Before th view
  findThemePath();

  initDoc();
  initView();

  //after the view
  initLevels();

  // To start the event loop first
  // QTimer::singleShot(0,mDoc,SLOT(startIntro()));
  QTimer::singleShot(0,this,SLOT(slotStartIntro()));
}

ZauralignApp::~ZauralignApp()
{
  mDoc->WriteConfig();
  #ifdef ZLOG
  mFile.close();
  #endif
  delete mConfig;

}

void ZauralignApp::findThemePath()
{
  QString name;
  mConfig->setGroup("Parameter");
  mTheme=mConfig->readEntry("Theme",QString("Default"));
  QString path=QString("%1pics/zauralign/").arg(mApp->qpeDir());
  QDir gDir;
  gDir.setPath(path);
  mThemeList=gDir.entryList(QString("theme_*"));
  for ( QStringList::Iterator it = mThemeList.begin(); it != mThemeList.end(); ++it )
  {
    name=(*it).mid(6);
    // Ok theme is still present
    if (mTheme == name)
    {
      break;
    }
  }
  // Chose any as current theme
  mTheme=name;
  mGrafixDir=path+QString("theme_")+name+QString("/");
}
void ZauralignApp::initThemes()
{

  for ( QStringList::Iterator it = mThemeList.begin(); it != mThemeList.end(); ++it )
  {
    QString name=(*it).mid(6);
    QAction *action= new QAction(name ,name, 0, settingsTheme, "settingsTheme" );
    action->setToggleAction(true);
    action->addTo(themeMenu);
    if (name == mTheme)
    {
      action->setOn(true);
    }
  }
}

void ZauralignApp::initLevels()
{
  QStringList lvl;
  mDoc->getLevels(lvl,0);
  for ( QStringList::Iterator it = lvl.begin(); it != lvl.end(); ++it )
  {
    QAction *action;
    action= new QAction(*it ,*it, 0, settingsLevel, "settingsLevel" );
    action->setToggleAction(true);
    action->addTo(levelMenu);
    if (*it == mDoc->level()) action->setOn(true);
  }
}

/** initializes all QActions of the application */
void ZauralignApp::initActions(){

  fileNew = new QAction(tr("New Game"), tr("&New"),
                        Qt::Key_Space, this);
  #ifdef ONLINE_HELP
  fileNew->setStatusTip(tr("Start a new Game"));
  fileNew->setWhatsThis(tr("New Game\n\nStart a new Game"));
  #endif
  fileNew->setEnabled(false);
  connect(fileNew, SIGNAL(activated()), this, SLOT(slotFileNew()));

  fileEnd = new QAction(tr("End Game"), tr("&End Game"), 0, this);
  #ifdef ONLINE_HELP
  fileEnd->setStatusTip(tr("Aborts a running game"));
  fileEnd->setWhatsThis(tr("End Game\n\nAborts a running game"));
  #endif
  fileEnd->setEnabled(false);
  connect(fileEnd, SIGNAL(activated()), this, SLOT(slotFileEnd()));

  fileHighscore = new QAction(tr("Show Highscores..."),
                              tr("&Show Highscores..."),
                              QAccel::stringToKey(tr("h")), this);
  #ifdef ONLINE_HELP
  fileHighscore->setStatusTip(tr("Shows the highscore dialog"));
  fileHighscore->setWhatsThis(tr("Show Highscores.\n\nShows the highscore dialog."));
  #endif
  connect(fileHighscore, SIGNAL(activated()), this, SLOT(slotFileHighscore()));

  fileQuit = new QAction(tr("Exit"), tr("E&xit"), 0, this);
  #ifdef ONLINE_HELP
  fileQuit->setStatusTip(tr("Quits the application"));
  fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
  #endif
  connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));



  settingsLevel = new QActionGroup(this);
  #ifdef ONLINE_HELP
  settingsLevel->setStatusTip(tr("Cuts the selected section and puts it to the clipboard"));
  settingsLevel->setWhatsThis(tr("Cut\n\nCuts the selected section and puts it to the clipboard"));
  #endif
  settingsLevel->setExclusive( true );
  connect( settingsLevel, SIGNAL( selected( QAction* ) ), this, SLOT( slotSettingsLevel( QAction* ) ) );

  settingsTheme = new QActionGroup(this);
  #ifdef ONLINE_HELP
  settingsTheme->setStatusTip(tr("Cuts the selected section and puts it to the clipboard"));
  settingsTheme->setWhatsThis(tr("Cut\n\nCuts the selected section and puts it to the clipboard"));
  #endif
  settingsTheme->setExclusive( true );
  connect( settingsTheme, SIGNAL( selected( QAction* ) ), this, SLOT( slotSettingsTheme( QAction* ) ) );

  helpAboutApp = new QAction(tr("About"), tr("&About..."),
                             QAccel::stringToKey("a"), this);
  #ifdef ONLINE_HELP
  helpAboutApp->setStatusTip(tr("About the application"));
  helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
  #endif
  connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));

}

void ZauralignApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR

  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
  fileMenu=new QPopupMenu();
  fileNew->addTo(fileMenu);
  fileEnd->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileHighscore->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileQuit->addTo(fileMenu);

  ///////////////////////////////////////////////////////////////////
  // submenu level
  levelMenu=new QPopupMenu();

  ///////////////////////////////////////////////////////////////////
  // submenu theme
  themeMenu=new QPopupMenu();

  ///////////////////////////////////////////////////////////////////
  // menuBar entry settingsMenu
  settingsMenu=new QPopupMenu();
  settingsMenu->insertItem(tr("Choose &level"),levelMenu);
  settingsMenu->insertItem(tr("Choose &theme"),themeMenu);
  //settingsTheme->addTo(settingsMenu);


  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  helpMenu=new QPopupMenu();
  helpAboutApp->addTo(helpMenu);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  menuBar()->insertItem(tr("&File"), fileMenu);
  menuBar()->insertItem(tr("&Settings"), settingsMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(tr("&Help"), helpMenu);

}



void ZauralignApp::initDoc()
{
  mDoc=new GameDoc(this);
  connect(mDoc,SIGNAL(gameOver(int)),this,SLOT(slotGameOver(int)));
  mDoc->setConfig(mConfig);
  mDoc->ReadConfig();
  
  // gives out status message
  // connect(mDoc,SIGNAL(statusMove(const QString &)),this,SLOT(slotStatusMsg(const QString &)));
}

void ZauralignApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // set the main widget here
  #ifdef DEBUG  
  cout << "DIR="<<mGrafixDir<<endl;
  #endif
  view=new GameView(mGrafixDir,this);
  mDoc->setView(view);
  setCentralWidget(view);
  //setCaption(0);
  setFocusPolicy( StrongFocus );
  //setFixedSize(QSize(240,300));

}

bool ZauralignApp::queryExit()
{
  mDoc->WriteConfig();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void ZauralignApp::slotFileNew()
{
  mDoc->newGame();
  settingsLevel->setEnabled( false );
  fileNew->setEnabled( false );
  fileEnd->setEnabled( true );
  #ifdef DEBUG
  cout << "slotFileNew" << endl;
  #endif
}

void ZauralignApp::slotFileEnd()
{
  mDoc->endGame();
  slotGameOver(-1);
}


void ZauralignApp::slotFileHighscore()
{
  mDoc->showHighscores(-1);
}

void ZauralignApp::slotFileQuit()
{
    #ifdef DEBUG
    cout << "slotFILEQUIT" << endl;
    #endif
    mApp->quit();
}

void ZauralignApp::slotSettingsTheme(QAction *action)
{
  if (!action) return;
  QString name=action->text();
  QString basepath=QString("%1pics/zauralign/").arg(mApp->qpeDir());
  QString path=basepath+QString("theme_")+name+QString("/");
  mConfig->setGroup("Parameter");
  mConfig->writeEntry("Theme",name);
  #ifdef DEBUG
  cout << "##### App:SlotSettingsTheme "<<action->text()<< "path="<<path<<endl;
  #endif
  mDoc->changeTheme(path);
}

void ZauralignApp::slotSettingsLevel(QAction *action)
{
  if (!action) return;
  mDoc->setLevel(action->text());
  #ifdef DEBUG
  cout << "settigns level "<<action->text()<< "doc="<<mDoc->level()<<endl;
  #endif

}

void ZauralignApp::slotHelpAbout()
{
  QMessageBox::about(this,
         tr("About..."),
         tr("Zauralign\nA nice desktop game for\none player.\n\n"
            "Align pieces of the same type until\nthey are removed.\n\n"
            "Version " VERSION "\n(c) 2001 by Martin Heni") );
}

/** No descriptions */
GameDoc * ZauralignApp::getDocument()
{
  return mDoc;
}
    
void ZauralignApp::slotGameOver(int mode)
{
  #ifdef DEBUG
  cout << "App:gameover slot"<<mode<<endl;
  #endif
  settingsLevel->setEnabled( true );
  fileNew->setEnabled(true);
  fileEnd->setEnabled(false);
}
void ZauralignApp::slotStartIntro()
{  
//  mDoc->startIntro();
  // Themes need to be here as they load the graphics
  // and start the intro
  initThemes();
  fileNew->setEnabled(true);
  fileEnd->setEnabled(false);
  #ifdef DEBUG
  cout << "Start intro"<<endl;
  #endif
}
