
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qdir.h>
#include <qimage.h>
#include <qmessagebox.h>
#include <math.h>
#include "zauralignview.h"
#include "zauraligndoc.h"
#include "zauralign.h"

#ifndef NO_DEBUG
#include <stream.h>
#include <streambuf.h>
#define DEBUG
#endif


// offset to avoid arm unsigned char
#define ARM_CHAR 64

#define DRAW_INTROTEXT
#define PRELOAD_GRAPHICS
#undef SHOW_ALL_PIECES
#undef NEWPIECE_VERTICAL
#undef QT_BUGFIX

#define COL_STATUSDARK   QColor(0,0,65)
#define COL_STATUSLIGHT  QColor(210,210,255)
#define COL_PLAYER       QColor(255,255,0)
#define COL_BACKGROUND   QColor(0,0,0)
#define COL_TEXT         QColor(255,200,0)

// Signals of the signal emitter:
// 1: move finished
// 2: animation finished



GameView::GameView(QString grafixdir,QWidget *parent, const char *name) : QCanvasView(0,parent, name)
{
  setVScrollBarMode(AlwaysOff);
  setHScrollBarMode(AlwaysOff);

  QDir dir(grafixdir);
  setGrafixDir(dir.absPath()+QString("/"));
  #ifdef DEBUG
  cout << "path="<<grafixdir<< " --->"<< mGrafix<<endl;
  #endif
  setBackgroundMode(PaletteBase);
  move(0,0);
  adjustSize();
  mCanvas=new QCanvas(parent);
  mCanvas->resize(parent->width(),parent->height());
  mCanvas->setDoubleBuffering(true);
  setCanvas(mCanvas);

  mIntroText=0;

  mConfig=0;
  initView();
  mSpriteDict.clear();
  //mItemDict.setAutoDelete(true);
  mSpriteDict.setAutoDelete(true);
  mItemDict.clear();

  QPixmap background; // empty pixmap sets default
  setBackground(background);

  #ifdef DEBUG
  cout << " ******INIT DONE ****************************** "<<endl;
  #endif


}

void GameView::initView(bool deleteall)
{
  QPoint defaultoffset;

  QString file=mGrafix+QString("grafix.rc");

  if (mConfig) delete mConfig;
  mConfig=new KConfig(file);
  if (!mConfig->hasGroup("game"))
  {
    #ifdef DEBUG
    cout << "ERROR: Could not find grafix data file " << file << endl;
    #endif
  }
  mConfig->setGroup("game");


  // TODO: Check for existance of file. if not warning and exit
  // checkv == ZAURALIGN
  QString checkv=mConfig->readEntry("VERSION");
  if (checkv != QString("ZAURALIGN"))
  {
    QMessageBox::critical(parentWidget(),tr("Zauralign"),
                    tr("Graphics data not found or wrong\n"
                       "version.\n"
                       "The game will probably not run!\n"
                       "Install it properly!\n"),
                    QMessageBox::Ok,QMessageBox::NoButton);
  }
  #ifdef DEBUG
  cout << "************** Game:HALLO="<<checkv<<endl;
  #endif
  
  int period=mConfig->readNumEntry("advanceperiod",0);
  mDecoration=mConfig->readListEntry("decoration");
  #ifdef DEBUG
  cout << " intiView mDecorations=" << mDecoration.count() << endl;
  cout << " intiView advance=" << period << endl;
  #endif

  //  mCanvas->setUpdatePeriod(25);
  if (period>0)
  {
    mCanvas->setAdvancePeriod(period);
  }

  mScoreDigits=4;

  boardSize=pieceSize*getDocument()->boardSize();
  defaultoffset=QPoint(10,10);
  boardOffset=mConfig->readPointEntry(QString("boardoffset"),&defaultoffset);

  defaultoffset=QPoint(10,90);
  newOffset=mConfig->readPointEntry("newoffset",&defaultoffset);

  defaultoffset=QPoint(10,30);
  scoreOffset=mConfig->readPointEntry("scoreoffset",&defaultoffset);

  defaultoffset=QPoint(0,0);
  gameoverOffset=mConfig->readPointEntry("gameoveroffset",&defaultoffset);
  defaultoffset=QPoint(0,0);
  gameoverDestination=mConfig->readPointEntry("gameoverdestination",&defaultoffset);

  QValueList<int> colorList;
  colorList=mConfig->readIntListEntry("textcolor");
  if (colorList.count()==3)
    mTextColor=QColor(colorList[0],colorList[1],colorList[2]);
  else
    mTextColor=QColor(COL_TEXT);

  // Careful: from here we change the config group!!
  // Get size of a QPixmap field
  QString name;
  QPixmap *sizePix;
  mConfig->setGroup("boardpiece");
  name=mConfig->readEntry("file");
  sizePix=loadPixmap(name);
  pieceSize=sizePix->size();
  delete sizePix;

  mConfig->setGroup("number");
  name=mConfig->readEntry("loadfile");
  name.sprintf(name.latin1(),0);
  sizePix=loadPixmap(name);
  numberSize=sizePix->size();
  delete sizePix;

  if (deleteall) deleteAllItems();
  #ifdef DEBUG
  cout << "+++++ Init view done " << endl;
  #endif
}

GameView::~GameView()
{
  if (mCanvas) delete mCanvas;
  if (mConfig) delete mConfig;
}
void GameView::setBackground(QPixmap &background)
{
  setBackgroundColor(COL_BACKGROUND);
  if (!mCanvas) return ;
  mCanvas->setBackgroundColor(COL_BACKGROUND);
  if (background.isNull())
  {
    #ifdef DEBUG
    cout << "Setting default background" << endl;
    #endif
    mConfig->setGroup("game");
    QString name=mConfig->readEntry("defaultbackground");
    QPixmap *pm=loadPixmap(name);
    if (pm && !pm->isNull()) background=*pm;
  }
  if (!background.isNull())
  {
    mCanvas->setBackgroundPixmap(background);
  }
  else
  {
    #ifdef DEBUG
    cout << "WARNING: Background pixmap not defined" << endl;
    #endif
  }
}

void GameView::changeTheme(QString s,bool deco)
{
  #ifdef DEBUG
  cout << "!!!!!GameView::changetheme="<<s<<endl;
  #endif

  QString key;
  if (deco) drawDecoration(true);

  PSprite *sprite;
  QDict<QCanvasItem> mTempDict(mItemDict);


  // Remove all sprites..and free memory
  QDictIterator<QCanvasItem> it( mTempDict );
  while ( it.current() )
  {
    key=it.currentKey();
    sprite=(PSprite *)(mItemDict.take(key));
    if (sprite) delete sprite;
    ++it;
  }
  mSpriteDict.clear();

  setGrafixDir(s);
  initView(false);

  QPixmap background; // empty pixmap sets default
  setBackground(background);

  // NEW
  preloadGraphics();
}

void GameView::Debug()
{
  #ifdef DEBUG
  cout << "Canvassize " << mCanvas->width() << " " << mCanvas->height() << endl;
  cout << "Items on canvas " << mCanvas->allItems().count() << endl;
  cout << "Spritecache size=" << mSpriteDict.count() << endl;
  QCanvasItemList list=mCanvas->allItems();
  int cnt=0,acnt=0;
 for (QCanvasItemList::ConstIterator it=list.begin(); it!=list.end(); ++it)
 {
    QCanvasItem *emp=*it;
    cout << "Item("<<cnt<<") " << emp << " animated=" << emp->animated() << " show=" << emp->visible()  << " active=" << emp->active() << endl;
    if (emp->animated()) acnt++;
    cnt++;
  }
  cout << "Item:" << cnt << " animated=" << acnt << endl;

  QDictIterator<QCanvasItem> it( mItemDict );
  cout  << "Sprite items in cache=" << mItemDict.size() << endl;
  while ( it.current() )
  {
    QCanvasItem *item=it.current();
    cout << it.currentKey() << endl;
    ++it;
  }
  #endif
}

GameDoc *GameView::getDocument() const
{
  ZauralignApp *theApp=(ZauralignApp *) parentWidget();
  return theApp->getDocument();
}

void GameView::drawBoard(int size,int newsize,bool remove)
{
  int x,y;
  QPoint origin;
  QCanvasItem *sprite;

  for (y=0;y<size;y++)
  {
    for (x=0;x<size;x++)
    {

      if (remove)
      {
        deleteItem("boardpiece",x+size*y);
        sprite=0;
      }
      else sprite=getItem("boardpiece",x+size*y);
      origin=boardOffset+QPoint(x*pieceSize.width(),y*pieceSize.height());
      if (sprite)
      {
        sprite->move(origin.x(),origin.y());
        sprite->show();
      }
    }
  }

  x=0;
  for (y=0;y<newsize;y++)
  {
    if (remove)
    {
      deleteItem("newpiece",y);
      sprite=0;
    }
    else sprite=getItem("newpiece",y);
    #ifdef NEWPIECE_VERTICAL
    origin=newOffset+QPoint(x*pieceSize.width(),y*pieceSize.height());
    #else
    origin=newOffset+QPoint(y*pieceSize.width(),x*pieceSize.height());
    #endif

    if (sprite)
    {
      sprite->move(origin.x(),origin.y());
      sprite->show();
    }
  }
}

QCanvasItem * GameView::drawPiece(int x,int y,char col,int no,int mode)
{
  QPoint origin;
  if (mode==1)
  {
    #ifdef NEWPIECE_VERTICAL
    origin=newOffset+QPoint(x*pieceSize.width(),y*pieceSize.height());
    #else
    origin=newOffset+QPoint(y*pieceSize.width(),x*pieceSize.height());
    #endif
  }
  else
  {
    origin=boardOffset+QPoint(x*pieceSize.width(),y*pieceSize.height());
  }

  QString name;
  name=QString("piece%1").arg((int)col);
  QCanvasItem *sprite=getItem(name,no);
  if (sprite)
  {
    sprite->move(origin.x(),origin.y());
    // TODO sprite->show();
  }
  return sprite;
}


void GameView::drawDecoration(bool remove)
{

  #ifdef DRAW_INTROTEXT
  if (mIntroText)
  {
    delete mIntroText;
    mIntroText=0;
  }
  #endif
  
  PSprite *sprite;

  for ( QStringList::Iterator it = mDecoration.begin(); it != mDecoration.end(); ++it )
  {
    int center=0;
    mConfig->setGroup(*it);
    unsigned int align=mConfig->readNumEntry("align",0);
    QPoint origin;
    if (align==1) origin=boardOffset;
    else if (align==2) origin=boardOffset+QPoint(boardSize.width(),0);
    else if (align==3) origin=boardOffset+QPoint(0,boardSize.height());
    else if (align==4) origin=boardOffset+QPoint(boardSize.width(),boardSize.height());
    else if (align==5) {origin=boardOffset+QPoint(boardSize.width()/2,0);center=1;}
    else if (align==6) {origin=boardOffset+QPoint(boardSize.width(),boardSize.height()/2);center=2;}
    else if (align==7) {origin=boardOffset+QPoint(boardSize.width()/2,boardSize.height());center=3;}
    else if (align==8) {origin=boardOffset+QPoint(0,boardSize.height()/2);center=4;}
    else origin=QPoint(0,0);

    #ifdef DEBUG
    cout  << "Decoration:"<< *it << " align=" << align << " center=" << center << " remove=" << remove <<  endl;
    #endif

    if (remove)
    {
      deleteItem(*it);
      sprite=0;
    }
    else sprite=(PSprite *)getItem(*it);
    if (sprite)
    {
      if (center==1) sprite->move(origin.x()-sprite->width()/2,origin.y()-sprite->height()); // x center
      else if (center==2) sprite->move(origin.x(),origin.y()-sprite->height()/2); // y center
      else if (center==3) sprite->move(origin.x()-sprite->width()/2,origin.y()); // x center
      else if (center==4) sprite->move(origin.x()-sprite->width(),origin.y()-sprite->height()/2); // y center
      else sprite->move(origin.x(),origin.y());
      sprite->show();
      // cout << "sprite=("<<sprite->x()<<","<<sprite->y()<<endl;
    }
  }

}

QCanvasItem * GameView::drawGameOver(int /*sequence*/,bool movement,bool remove)
{
  PSprite *sprite;

   #ifdef DEBUG
   cout << "Items on canvas " << mCanvas->allItems().count() << endl;
   #endif

  if (remove)
  {
    deleteItem("gameover");
    sprite=0;
  }
  else sprite=(PSprite *)getItem("gameover");
  if (sprite)
  {
    int w=sprite->pixmap()->width();
    int dx=boardOffset.x()+(boardSize.width()-w)/2;
    if (movement)
    {
      sprite->move(dx+gameoverOffset.x(),gameoverOffset.y());
      sprite->moveTo(dx+gameoverDestination.x(),gameoverDestination.y());
    }
    else
    {
      sprite->move(dx+gameoverDestination.x(),gameoverDestination.y());
    }
    sprite->setAnimation(0);
    sprite->show();
  }
  return sprite;
}

void GameView::preloadGraphics()
{
#ifdef PRELOAD_GRAPHICS
  ZauralignApp *theApp=(ZauralignApp *) parentWidget();

  QCanvasText *text=new QCanvasText(mCanvas);
  text->setColor(mTextColor);
  text->setText(tr("Please wait...preloading graphics"));
  text->move(40,50);
  text->show();

  PSprite *sprite;
  mConfig->setGroup("game");
  bool preload=mConfig->readBoolEntry("preload",true);
  if (preload)
  {
    for (unsigned int i=1;i<=8;i++)
    {
      QString s=QString("piece%1").arg(i);
      sprite=(PSprite *)getItem(s,999);
      if (sprite)
      {
        sprite->move(5+sprite->width()*i,100);
        sprite->show();
      }
      //deleteItem(s,999);
      theApp->application()->processEvents();
    }

    //KApplication::kApplication()->processEvents();
    sprite=(PSprite *)getItem("gameover",999);
    deleteItem("gameover",999);

    sprite=(PSprite *)getItem("number",999);
    deleteItem("number",999);
  }
  // Clean up
  delete text;
  for (unsigned int i=1;i<=8;i++)
  {
    QString s=QString("piece%1").arg(i);
    deleteItem(s,999);
  }

  #endif
}
void GameView::drawIntro(bool /*remove*/)
{
  #ifdef DEBUG
  cout << "!!!!!GameView::drawIntro="<<endl;
  #endif
  PSprite *sprite;

  // NEW preloadGraphics();

#ifdef SHOW_ALL_PIECES
  for (unsigned int i=1;i<=8;i++)
  {
    QString s=QString("piece%1").arg(i);
    sprite=(PSprite *)getItem(s,999);
    sprite->setAnimation(0);
    sprite->move(0,pieceSize.width()*(i-1));
    sprite->show();
  }
#endif

  #ifdef DRAW_INTROTEXT
  if (!mIntroText)
  {
    mIntroText=new QCanvasText(mCanvas);
    mIntroText->setColor(mTextColor);
    mIntroText->setText(tr("Press SPACE to start a new game"));
    mIntroText->move(40,10);
    mIntroText->show();
  }
  #endif

  mConfig->setGroup("game");
  double introspeed=mConfig->readDoubleNumEntry("introspeed",0);
  QString name=mConfig->readEntry("defaultintro");

  QPixmap *pixmap=loadPixmap(name);
  if (!pixmap || pixmap->isNull())
  {
    #ifdef DEBUG
    cout << "WARNING: No intropicture defined."<<endl;
    #endif
    return ;
  }
  QImage img=pixmap->convertToImage();  // slow
  GameDoc *doc=getDocument();
  int no;

  for (int y=0;y<img.height();y++)
  {
    for (int x=0;x<img.width();x++)
    {
      QColor col(img.pixel(x,y));
      if (col.red()>127 && col.green()>127 && col.blue()>127) no=1;
      else if (col.red()>127 && col.green()>127 ) no=3;
      else if (col.red()>127 && col.blue()>127) no=7;
      else if (col.green()>127 && col.blue()>127) no=5;
      else if (col.red()>127) no=2;
      else if (col.green()>127) no=4;
      else if (col.blue()>127) no=6;
      else no=0;
      QString s=QString("piece%1").arg(no);
      if (no>0)
      {
        int r0=doc->random(4);
        int x0=doc->random(100)-50;
        int y0=doc->random(100)-50;
        if (r0==1) // TR
        {
          x0+=contentsWidth();
        }
        else if (r0==2) // BL
        {
          y0+=height();
        }
        if (r0==3) // BR
        {
          x0+=contentsWidth();
          y0+=contentsHeight();
        }
        sprite=(PSprite *)getItem(s,x+y*img.width());
        sprite->setZ(x+y*img.width());
        sprite->move(x0,y0);
        sprite->moveTo((pieceSize.width()/2)*x,20+(pieceSize.height()/2)*y);
        if (introspeed>0.0) sprite->setSpeed(introspeed);
        sprite->setAnimation(2,2.0);
        sprite->show();
      }
    }
  }

  #ifdef DEBUG
  cout << "Items on canvas after intro " << mCanvas->allItems().count() << endl;
  #endif


  QTimer::singleShot(7000,this,SLOT(slotContinueIntro()));
}

void GameView::slotContinueIntro()
{
  static int rcnt=0;
  static int scnt=0;
  if (!getDocument()->isIntro()) return ;

  QCanvasItemList list=mCanvas->allItems();

  int anim=0;
  double mod=1.0;
  if (rcnt==0)
  {
    anim=1;
    mod=1.5;
    QTimer::singleShot(3000,this,SLOT(slotContinueIntro()));
    scnt=0;
  }
  else if (rcnt==1)
  {
    anim=4;
    QTimer::singleShot(70,this,SLOT(slotContinueIntro()));
  }
  else if (rcnt==2)
  {
    anim=0;
    QTimer::singleShot(3500,this,SLOT(slotContinueIntro()));
  }
  else if (rcnt==3)
  {
    anim=2;
    mod=1.5;
    QTimer::singleShot(7000,this,SLOT(slotContinueIntro()));
  }


  int cnt=0;
  for (QCanvasItemList::ConstIterator it=list.begin(); it!=list.end(); ++it)
  {
      // Only sprites !
      if ( (*it)->rtti()!=1000) continue;

      PSprite *sprite=(PSprite *)(*it);

      if (anim==0)
      {
        sprite->hide();
      }
      else if (anim==4 && cnt==scnt-5)
      {
        sprite->hide();
      }
      else if (anim==4 && cnt!=scnt)
      {
      }
      else
      {
        if (!sprite->visible()) sprite->show();
        sprite->setAnimation(anim,mod);
      }

      cnt++;
  }

  if (anim==4)
  {
    scnt++;
    if (scnt==(int)list.count()) rcnt++;
  }
  else
  {
    rcnt++;
  }
  if (rcnt>3) rcnt=0;

}

void GameView::drawScore(bool remove)
{
  int score=getDocument()->score();
  /*
  if (global_debug==42) score+=9990;
  if (global_debug==4242) score+=99990;
  if (global_debug==424242) score+=999990;
  */

  // Check how many digits
  int dgScore=1;
  int digits;
  int i;
  for (digits=0;digits<10;digits++)
  {
    if (score<dgScore) break;
    dgScore*=10;
  }
  //kdDebug() << " score=" << score << " dgScore=" << dgScore << " digits=" << digits << endl;
  if (digits>mScoreDigits)
  {
    #ifdef DEBUG
    cout << "We need more digits" << endl;
    #endif
    mScoreDigits=digits;
  }

  char no[mScoreDigits];
  for (i=0;i<mScoreDigits;i++)
  {
    no[mScoreDigits-i-1]=score%10;
    score/=10;
  }

  QPoint borigin;
  QPoint sorigin;
  QPoint offset;

  sorigin=scoreOffset;
  borigin=scoreOffset;

  PSprite *numberSprite;
  PSprite *backSprite;
  for (int i=0;i<mScoreDigits;i++)
  {

    QString b;
    if (i==0) b=QString("score_left");
    else if (i==mScoreDigits-1) b=QString("score_right");
    else b=QString("score_middle");
    QString s=QString("number");
    if (remove)
    {
      deleteItem(s,i);
      numberSprite=0;
      if (i==mScoreDigits-1) deleteItem(b,0);
      else deleteItem(b,i);
      backSprite=0;
    }
    else
    {
      numberSprite=(PSprite *)getItem(s,i);
      if (i==mScoreDigits-1) backSprite=(PSprite *)getItem(b,0);
      else backSprite=(PSprite *)getItem(b,i);
    }

    if (backSprite && numberSprite)
    {
      backSprite->move(borigin.x(),borigin.y());
      backSprite->show();

      offset=QPoint((backSprite->width()-numberSprite->width())/2,(backSprite->height()-numberSprite->height())/2);

      numberSprite->setFrame(no[i]);
      numberSprite->move(sorigin.x()+offset.x(),sorigin.y()+offset.y());
      numberSprite->show();

      sorigin+=QPoint(numberSprite->width(),0);
      borigin+=QPoint(backSprite->width(),0);
    }
  }
}


void GameView::resizeEvent(QResizeEvent *e)
{
  if (mCanvas) mCanvas->resize(e->size().width(),e->size().height());
}



// mouse click event
void GameView::contentsMousePressEvent( QMouseEvent *mouse )
{
  if (mouse->button()!=LeftButton) return ;
  QPoint point;
  point=mouse->pos();
  point-=boardOffset;
  if (point.x()<0 || point.y()<0) return ;
  if (point.x()>=boardSize.width() || point.y()>=boardSize.height()) return ;


  int movex,movey;
  movex=point.x()/pieceSize.width();
  movey=point.y()/pieceSize.width();
//  kdDebug() << "Move to " << movex << " , " << movey << endl;

  GameDoc *doc=getDocument();
  if (!doc->isRunning())
  {
    #ifdef DEBUG
    cout << "Game not running"<<endl;
    #endif
    return ;
  }

  // Move all to gamedoc!!!!
  if (doc->moveStatus()==GameDoc::MoveEnd) return ; // new move not yet possible
  else if (doc->moveStatus()==GameDoc::NoMove)
  {
    if (doc->getBoard(movex,movey)!=0)
    {
      doc->firstPiece(movex,movey);
    }
    #ifdef DEBUG
    else
      cout << "No piece here" <<endl;
    #endif

  }
  else if (doc->moveStatus()==GameDoc::MoveStart)
  {
    // TODO Check for move possible
    if (doc->getBoard(movex,movey)!=0)
    {
      // Perform first move
      doc->firstPiece(movex,movey);
    }
    else
    {
      if (!doc->secondPiece(movex,movey))
      {
        #ifdef DEBUG
        cout << "No path found" << endl;
        #endif
      }
      else
      {
        QString s=QString("piece%1").arg(doc->moveColour());
        PSprite *sprite=(PSprite *)(getItem(s,doc->moveSprite()));
        sprite->setAnimation(0);
        connect(sprite->emitter(),SIGNAL(signalNotify(QCanvasItem *,int)),
                this,SLOT(slotMoveTo(QCanvasItem *,int)));
        slotMoveTo(sprite,1);
      }
    }
  }

}
void GameView::slotMoveTo(QCanvasItem *sprite,int mode)
{
  if (!sprite) return ;
  if (mode!=1)
  {
    #ifdef DEBUG
    cout << "Wrong mode in slotMoveTo -> returning "<<endl;
    #endif
    return ;
  }
  GameDoc *doc=getDocument();

  int x=0,y=0;
  ListData *waypoint=doc->waypoint();
  if (waypoint)
  {
    x=waypoint->x;
    y=waypoint->y;
  }

  if (!waypoint)
  {
    doc->moveFinished();
    QObject *ob=((PSprite *)sprite)->emitter(false);
    if (ob) disconnect(ob,SIGNAL(signalNotify(QCanvasItem *,int)),
                       this,SLOT(slotMoveTo(QCanvasItem *,int)));
    ((PSprite *)sprite)->deleteEmitter();
  }
  else
  {
    QPoint origin;
    origin=boardOffset+QPoint(x*pieceSize.width(),y*pieceSize.height());
    ((PSprite *)sprite)->moveTo(origin.x(),origin.y());
  }
  doc->removeWaypoint(waypoint);
}



QPixmap * GameView::loadPixmap(QString file,QString mask)
{
  #ifdef DEBUG
  //cout << "loadPixmap "<<mGrafix << "::"<<file<<endl;
  #endif
  QPixmap *newP=new QPixmap;
  bool result1=false;
  if (!file.isNull())
  {
    result1=newP->load(mGrafix+file);
  }
  #ifdef USE_PIXMAP_MASK
  bool result2=false;
  if (result1 && !mask.isNull())
  {
    QBitmap bitmask;
    if (mask==QString("auto"))  // self mask..slow but quick to create
    {
      newP->setMask( newP->createHeuristicMask() );
      result2=true;
    }
    else
    {
      result2=bitmask.load(mGrafix+mask);
      if (result2) newP->setMask(bitmask);
    }
  }
  #endif
  #ifdef DEBUG
  //cout << "GameView::loadPixmap: file="<<file<<"   mask="<<mask<<"   result1="<<result1<<"   result2="<<result2<<endl;
  #endif

  return newP;
}

void GameView::deleteAllItems()
{
  QDictIterator<QCanvasItem> it( mItemDict );
  while ( it.current() )
  {
    QCanvasItem *item=it.current();
    mItemDict.remove(it.currentKey());
    delete item;
  }
}
void GameView::deleteItem(QString s,unsigned int no)
{
  QCanvasItem *item;
  QString name=s+QString("_%1").arg(no);
  item=mItemDict[name];
  if (item)
  {
    mItemDict.remove(name);
    delete item;
  }
}

void GameView::deleteItem(QCanvasItem *item)
{
  QDictIterator<QCanvasItem> it( mItemDict );
  while ( it.current() )
  {
    if (item==it.current())
    {
      mItemDict.remove(it.currentKey());
      delete item;
      return ;
    }
    ++it;
  }
}

QCanvasItem * GameView::getItem(QString s,unsigned int no)
{
  QCanvasItem *item;
  QCanvasPixmapArray *pixmaparray;
  double z,movespeed;
  char anima[MAX_ANIM];
  char animb[MAX_ANIM];
  char animdir[MAX_ANIM];
  char animspeed[MAX_ANIM];
  unsigned int i;

  // cout << " -> getItem("<<s<<","<<no<<")"<<endl;
  item=mItemDict[s+QString("_%1").arg(no)];
  if (item)
  {
    //cout << "found item "<<s+QString("_%1").arg(no)<<" as directly existing "<<item << endl;
    return item;
  }

  // check whether we can copy a sprite
  SpriteCache *spriteCache=mSpriteDict[s];
  if (!spriteCache)
  {
    // Now first time load the items
    if (!mConfig->hasGroup(s))
    {
      #ifdef DEBUG
      cout << "Item "<<s <<" not defined !!! " <<endl;
      #endif
      return 0;
    }
    #ifdef ZLOG
    //cout << " +++++ Creating new item "<<s << " count in list="<<mItemDict.count()<< " no="<<no <<  endl;
    zlog << " +++++ Creating new item "<<s << " count in list="<<mItemDict.count()<< " no="<<no <<  endl;
    #endif
    mConfig->setGroup(s);
    movespeed=mConfig->readDoubleNumEntry("movespeed",10.0);
    QPoint defaultoffset=QPoint(0,0);
    QPoint offset=mConfig->readPointEntry("offset",&defaultoffset);
    QValueList<int> filterList;
    filterList=mConfig->readIntListEntry("colorfilter");
    QValueList<int> transformList;
    transformList=mConfig->readIntListEntry("transformfilter");
    QStringList operationList=mConfig->readListEntry("pixmaps");
    z=mConfig->readDoubleNumEntry("z",0.0);
    // Append default entryy
    if (operationList.count()==0)
    {
      QString s;
      operationList.append(s);
    }


    QValueList<int> animList;
    // frame A, frame B, mode, delay
    for (i=0;i<MAX_ANIM;i++)
    {
      QString animname=QString("anim%1").arg(i);
      animList=mConfig->readIntListEntry(animname);
      if (mConfig->hasKey(animname) && animList.count()>=4)
      {
        anima[i]=(char)animList[0];
        animb[i]=(char)animList[1];
        // char on ARM seems to be unsigned
        animdir[i]=(char)(animList[2]+ARM_CHAR);
        animspeed[i]=(char)animList[3];
      }
      else
      {
        anima[i]=0;
        animb[i]=0;
        animdir[i]=ARM_CHAR;
        animspeed[i]=0;
      }
      #ifdef DEBUG
      //cout << "ANIM parameter loaded a="<<(int)anima[i]
      //     <<" b="<<(int)animb[i]<<" dir="<<(int)(animdir[i])<<" speed="<<(int)animspeed[i]<<endl;
      #endif
    }

    QPixmap *pixmap;
    QList<QPixmap> pixlist;
    pixlist.setAutoDelete(true);
    QList<QPoint> hotlist;
    hotlist.setAutoDelete(true);

    for ( QStringList::Iterator it = operationList.begin(); it !=operationList.end(); ++it )
    {
      QString name=*it;
      QString type=mConfig->readEntry(name+"type");
      if (type.isNull()) type=QString("load"); // default load
      //kdDebug() << " Processing operation " << (name.isNull()?"default":name) << "type="<<type << endl;
      unsigned int number=mConfig->readNumEntry(name+"number",1);
      // Load a given set of images or replace a %d by a sequence if there are
      // less image names than number given
      if (type==QString("load"))
      {
        QString pixfile=mConfig->readEntry(name+"file");
        QString maskfile=mConfig->readEntry(name+"mask");
        // Read images
        for (i=0;i<number;i++)
        {
          QString tmpfile,tmpmask;
          tmpfile.sprintf(pixfile.latin1(),i);
          tmpmask.sprintf(maskfile.latin1(),i);

          pixmap=loadPixmap(tmpfile,tmpmask);
          #ifdef ZLOG
          zlog << "  pixmap "<<tmpfile<< " pdepth="<<pixmap->depth()<<endl;
          #endif
          
          if (!pixmap)
          {
            #ifdef DEBUG
            cout << "Could not create pixmap="<<tmpfile << " with mask " << tmpmask << endl;
            #endif
            #ifdef ZLOG
            zlog << "Could not create pixmap="<<tmpfile << " with mask " << tmpmask << endl;
            #endif
          }
          else
          {
            if (transformList.count()>0)
            {
              if (transformList[0]==1 && transformList.count()==2)  // rotate
              {
                rotatePixmap(pixmap,transformList[1]);
              }
              else if (transformList[0]==2 && transformList.count()==3) // scale
              {
                scalePixmap(pixmap,(double)transformList[1]/100.0,(double)transformList[2]/100.0);
              }
              else
              {
                #ifdef ZLOG
                zlog << "Transform filter error " << tmpfile << endl;
                #endif
              }
            }
            
            // apply colorfilter
            if (filterList.count()>0)
            {
              // Only filter 1 HSV implemented
              if (filterList[0]==1 && filterList.count()==4) changeHSV(pixmap,filterList[1],filterList[2],filterList[3]);
              else if (filterList[0]==2 && filterList.count()==2) changeGrey(pixmap,filterList[1]);
              else if (filterList[0]==2 && filterList.count()==1) changeGrey(pixmap);
              #ifdef DEBUG
              else cout << "WARNING: Colorfilter parameter incorrect "<< endl;
              #endif
            }
            pixlist.append(pixmap);
            QPoint *copyoffset=new QPoint(-offset);
            hotlist.append(copyoffset);
          }
        }
      }
      // Scale some images in given axis
      else if (type==QString("scale"))
      {
        // scale images
        QString tmpfile=mConfig->readEntry(name+"file");
        QString tmpmask=mConfig->readEntry(name+"mask");
        int axis=mConfig->readNumEntry(name+"axis",1);
        int maxpiece=mConfig->readNumEntry(name+"max",number);
        #ifdef DEBUG
        cout << " Scaling " << number << " pics axis="<<axis<<" max="<<maxpiece<<" file="<<tmpfile<< " mask="<<tmpmask<<endl;
        #endif
        #ifdef ZLOG
        zlog << " Scaling " << number << " pics axis="<<axis<<" max="<<maxpiece<<" file="<<tmpfile<< " mask="<<tmpmask<<endl;
        #endif

        pixmap=loadPixmap(tmpfile,tmpmask);
        for (unsigned int j=0;j<(unsigned int)number;j++)
        {
          double sc=(double)(maxpiece-j)/(double)(maxpiece);
          QPixmap *copypixmap=new QPixmap(*pixmap);
          if (axis==1) scalePixmap(copypixmap,sc,1.0);
          else if (axis==2) scalePixmap(copypixmap,1.0,sc);
          else scalePixmap(copypixmap,sc,sc);
          
          // Unused
          if (transformList.count()>0)
          {
            if (transformList[0]==1 && transformList.count()==2)  // rotate
            {
              rotatePixmap(copypixmap,transformList[1]);
            }
            else if (transformList[0]==2 && transformList.count()==3) // scale
            {
              scalePixmap(copypixmap,(double)transformList[1]/100.0,(double)transformList[2]/100.0);
            }
          }
          // end unused

          // apply color filter
          if (filterList.count()>0)
          {
            // Only filter 1 HSV implemented
            if (filterList[0]==1 && filterList.count()==4) changeHSV(copypixmap,filterList[1],filterList[2],filterList[3]);
            else if (filterList[0]==2 && filterList.count()==2) changeGrey(copypixmap,filterList[1]);
            else if (filterList[0]==2 && filterList.count()==1) changeGrey(copypixmap);
            #ifdef DEBUG
            else cout << "WARNING: Colorfilter parameter incorrect "<< endl;
            #endif
          }
          #ifdef DEBUG
          cout << "  append as "<<pixlist.count()<<endl;
          #endif
          pixlist.append(copypixmap);
          QPoint *copyoffset=new QPoint((-pixmap->width()+copypixmap->width())/2,(-pixmap->height()+copypixmap->height())/2);
          hotlist.append(copyoffset);
        }
        delete pixmap;

      }
      else
      {
        #ifdef DEBUG
        cout << "WARNING: Unknown algorithm " << type << " for " << name << " not supported " << endl;
        #endif
      }
    }// end create images


    //kdDebug() <<"Pixarray count="<<pixlist.count()<<endl;
    if (pixlist.count()<1) return 0;

    #ifdef DEBUG
    cout << "Sprite "<<s<< "with "<<pixlist.count()<<" images" << endl;
    #endif
    pixmaparray=new QCanvasPixmapArray(pixlist,hotlist);
    #ifdef DEBUG
    if (!pixmaparray) cout << "Could not create pixmaparray" << endl;
    #endif
    spriteCache=new SpriteCache;
    spriteCache->pixmapArray=pixmaparray;
    spriteCache->z=z;
    spriteCache->mMoveSpeed=movespeed;
    for (i=0;i<MAX_ANIM;i++)
    {
      spriteCache->anima[i]=anima[i];
      spriteCache->animb[i]=animb[i];
      spriteCache->animdir[i]=animdir[i];
      spriteCache->animspeed[i]=animspeed[i];
    }
    mSpriteDict.insert(s,spriteCache);
  }
  else // retrieve fomr buffers
  {
    pixmaparray=spriteCache->pixmapArray;
    z=spriteCache->z;
    movespeed=spriteCache->mMoveSpeed;
    for (i=0;i<MAX_ANIM;i++)
    {
      anima[i]=spriteCache->anima[i];
      animb[i]=spriteCache->animb[i];
      animdir[i]=spriteCache->animdir[i];
      animspeed[i]=spriteCache->animspeed[i];
    }
    //kdDebug() << "Sprite "<<s<<" was cached. copying to " << no << endl;
  }

  PSprite *sprite=new PSprite(pixmaparray,mCanvas);
  sprite->setSpeed(movespeed*getDocument()->speedModify());
  sprite->setZ(z);
  for (i=0;i<MAX_ANIM;i++)
  {
    sprite->prepareAnimation(i,anima[i],animb[i],animdir[i],animspeed[i]);
  }
  sprite->setAnimation(0);
  item=sprite;
  #ifdef DEBUG
  //cout << "sprite"<<no<<" as '"<<s+QString("_%1").arg(no)<<"' inserted  (z="<<z<<") ok  and dictionary inserted ok" << endl;
  #endif
  mItemDict.insert(s+QString("_%1").arg(no),item);
  return item;
}

void GameView::setAlphaBuffer(QPixmap *pixmap,bool b)
{
  // The following is nonsense and only needed
  // because my QT version looses the alpha buffer
  // when doing transforms
  if (!pixmap || pixmap->isNull())
  {
    #ifdef ZLOG
    zlog << "Error in setting aklpha buffer "<<pixmap<<endl;
    #endif
    return;
  }
  #ifdef ZLOG
  zlog << "  vor alpha pdepth="<<pixmap->depth()<<endl;
  #endif
  QImage img=pixmap->convertToImage();  // slow
  img.convertDepth(32);
  img.setAlphaBuffer(b);
  pixmap->convertFromImage(img,QPixmap::Color); // slow
  #ifdef ZLOG
  zlog << "  img alpha="<<img.hasAlphaBuffer() << " pdepth="<<pixmap->depth()<< "idepth="<<img.depth()<<endl;
  #endif

}

void GameView::scalePixmap(QPixmap *pixmap,double sx,double sy)
{
  if (!pixmap || pixmap->isNull())
  {
    return ;
  }

  QImage src=pixmap->convertToImage();
  int w=src.width();
  int h=src.height();
  int th,tw;
  
  tw=(int)(w*sx);
  th=(int)(h*sy);

  // Create null pixmap
  if (tw<=0 || th<=0)
  {
    pixmap->resize(0,0);
    return ;
  }

  #ifdef ZLOG
  zlog << "Target w="<<tw<<" h="<<th<< " with scale" << sx<<","<<sy<<endl;
  #endif
  
  QImage dst(tw,th,src.depth());
  dst.setAlphaBuffer(src.hasAlphaBuffer());

  // Bugfix for QT Alpha problem
  #ifdef QT_BUGFIX
  int cnt=0;
  int xa=0,ya=0;
  QRgb pixa=0;
  #endif

  for (int y=0;y<th;y++)
  {
    for (int x=0;x<tw;x++)
    {
      QRgb pix;
      if (sx>0.0 && sy >0.0)
        pix=src.pixel((int)(x/sx),(int)(y/sy));
      else if (sx>0.0)
        pix=src.pixel((int)(x/sx),0);
      else if (sy>0.0)
        pix=src.pixel(0,(int)(y/sy));
      else
        pix=src.pixel(0,0);
      dst.setPixel(x,y,qRgba(qRed(pix),qGreen(pix),qBlue(pix),qAlpha(pix)));

      #ifdef QT_BUGFIX
      if (qAlpha(pix)!=0 && qAlpha(pix)!=255)
      {
        cnt++;
      }
      else if (qAlpha(pix)==0)
      {
        xa=x;
        ya=y;
        pixa=pix;
      }
      #endif
    }
  }
  #ifdef ZLOG
  zlog << "src pixmap=" << pixmap->width()<< ","<<pixmap->height()<<" depth="<<pixmap->depth()<<endl;
  zlog << "src img d="<<src.depth()<<" dst d="<<dst.depth()<<endl;
  #endif
  #ifdef QT_BUGFIX
  // Bugfix for QT Alpha problem . we need one not 0/255 pixel
  if  (cnt==0)
  {
    for (int y=0;y<th;y++)
    {
      for (int x=0;x<tw;x++)
      {
        QRgb pix;
        pix=dst.pixel(x,y);
        dst.setPixel(x,y,qRgba(qRed(pix),qGreen(pix),qBlue(pix),qAlpha(254)));
      }
    }
    #ifdef ZLOG
    zlog << "Setting Pixel "<<xa<<","<<ya<<" to alpha 1="<<254<<" pix="<<pixa<<endl;
    #endif
  }
  #ifdef ZLOG
  zlog << "Scale image: depth="<<pixmap->depth()<< " alpha cnt="<<cnt<<endl;
  #endif
  #endif
  pixmap->convertFromImage(dst);
  #ifdef ZLOG
  zlog << "dst pixmap=" << pixmap->width()<< ","<<pixmap->height()<<endl;
  zlog << "Scale image: depth="<<pixmap->depth()<< " alpha "<<endl;
  #endif
}

void GameView::rotatePixmap(QPixmap *pixmap,int deg)
{
  if (!pixmap || pixmap->isNull())
  {
    return ;
  }

  QImage src=pixmap->convertToImage();
  int w=src.width();
  int h=src.height();
  int th,tw;
  if (deg==180)
  {
    th=h;
    tw=w;
  }
  else if (deg==90 || deg==270)
  {
    th=w;
    tw=h;
  }
  else
  {
    #ifdef LOG
    zlog << "Calling rotatePixmap with not supported deg="<<deg<<endl;
    #endif
    return;
  }
  QImage dst(tw,th,src.depth());
  dst.setAlphaBuffer(src.hasAlphaBuffer());

  for (int y=0;y<h;y++)
  {
    for (int x=0;x<w;x++)
    {
      QRgb pix=src.pixel(x,y);
      if (deg==180)
        dst.setPixel(x,h-1-y,qRgba(qRed(pix),qGreen(pix),qBlue(pix),qAlpha(pix)));
      else if (deg==90)
        dst.setPixel(h-1-y,x,qRgba(qRed(pix),qGreen(pix),qBlue(pix),qAlpha(pix)));
      else if (deg==270)
        dst.setPixel(y,x,qRgba(qRed(pix),qGreen(pix),qBlue(pix),qAlpha(pix)));
    }
  }
  pixmap->convertFromImage(dst);
}

void GameView::changeHSV(QPixmap *pixmap,int dh,int ds,int dv)
{
  if (!pixmap || (dh==0 && ds==0 && dv==0)) return ;
  if (pixmap->isNull()) return ;
  if (pixmap->width()==0 && pixmap->height()==0) return ;

  int h,s,v;
  QColor black=QColor(0,0,0);
  QImage img=pixmap->convertToImage();  // slow

  for (int y=0;y<img.height();y++)
  {
    for (int x=0;x<img.width();x++)
    {
      QRgb pix=img.pixel(x,y);
      QColor col(pix);
      col.hsv(&h,&s,&v);
      if (col==black) continue;  // speed up?
      h=((unsigned int)(h+dh))%360;
      s=((unsigned int)(s+ds)%256);
      v=((unsigned int)(v+dv)%256);
      col.setHsv(h,s,v);
      img.setPixel(x,y,qRgba(col.red(),col.green(),col.blue(),qAlpha(pix)));
    }
  }
  pixmap->convertFromImage(img); // slow
}
void GameView::changeGrey(QPixmap *pixmap,int lighter)
{
  if (!pixmap) return ;
  if (pixmap->isNull()) return ;
  if (pixmap->width()==0 && pixmap->height()==0) return ;

  QImage img=pixmap->convertToImage();  // slow

  for (int y=0;y<img.height();y++)
  {
    for (int x=0;x<img.width();x++)
    {
      QRgb pix=img.pixel(x,y);
      int gray=qGray(qRed(pix),qGreen(pix),qBlue(pix));
      QColor col(gray,gray,gray);
      if (lighter>0) col=col.light(lighter);
      if (lighter<0) col=col.dark(-lighter);
      img.setPixel(x,y,qRgba(col.red(),col.green(),col.blue(),qAlpha(pix)));
    }
  }
  pixmap->convertFromImage(img); // slow
}



// ---------------- Sprite ------------
PSprite::PSprite(QCanvasPixmapArray *p,QCanvas *c) : QCanvasSprite(p,c)
{
  for (unsigned int i=0;i<MAX_ANIM;i++)
  {
    animb[i]=0;
    anima[i]=0;
    animdir[i]=0;
    animspeed[i]=0;
  }
  mAnimModify=1.0;
  animationNumber=0;
  animSpeedCnt=0;
  currentAnimDir=0;
  mEmitter=0;
  targetX=0;
  targetY=0;
  moveStatus=0;
  speed=1.0;
  mData=0;
}

PSprite::~PSprite()
{
  if (mEmitter) delete mEmitter;
}

int PSprite::width()
{
  //kdDebug() << "Width:"<< QCanvasSprite::width()<<endl;
  return QCanvasSprite::width();
}
int PSprite::height()
{
  //kdDebug() << "height:"<< QCanvasSprite::height()<<endl;
  return QCanvasSprite::height();
}

void PSprite::advance(int stage)
{
  //kdDebug()  << "PSprite::Advance beginmove status of " << this << " = " << moveStatus << " stage=" << stage << endl;
  if (stage!=1) return ;
  if (!visible()) return ;

  bool isAnim=true;
  bool isMoving=false;

  int emitsignal=0;
//  kdDebug() << "Advance for " << this << endl;

  // Animation
  // mode  0: no animation
  //       1: single shot a->b
  //      -1: single shot b->a
  //       2: cycle  a->b->a
  //      -2: cycle  b->a->b
  //       3: cycle  a->b
  //      -3: cycle  b->a
  animSpeedCnt++;
  if (animspeed[animationNumber]==0)
  {
    // nothign to do?
    //setFrame(anima[animationNumber]);
    isAnim=false;
    animSpeedCnt=0;
  }
  if (animSpeedCnt>=(int)(double)animspeed[animationNumber]*mAnimModify)
  {
    switch(((int)animdir[animationNumber])-ARM_CHAR)
    {
      case 1:
        if (frame()+1 <= animb[animationNumber]) setFrame(frame()+1);
        else emitsignal=2;
      break;
      case -1:
        if (frame()-1 >= anima[animationNumber]) setFrame(frame()-1);
        else emitsignal=2;
      break;
      case 2:
        if (((int)animdir[animationNumber])-ARM_CHAR==currentAnimDir)
        {
          if (frame()+1 <= animb[animationNumber]) setFrame(frame()+1);
          else
          {
            currentAnimDir=-currentAnimDir;
            if (frame()>0) setFrame(frame()-1);
          }
        }
        else
        {
          if (frame()-1 >= anima[animationNumber]) setFrame(frame()-1);
          else
          {
            currentAnimDir=-currentAnimDir;
            if (frame()+1<frameCount()) setFrame(frame()+1);
          }
        }
      break;
      case -2:
        if (((int)animdir[animationNumber])-ARM_CHAR==currentAnimDir)
        {
          if (frame()-1 >= anima[animationNumber]) setFrame(frame()-1);
          else
          {
            currentAnimDir=-currentAnimDir;
            if (frame()+1<frameCount()) setFrame(frame()+1);
          }
        }
        else
        {
          if (frame()+1 <= animb[animationNumber]) setFrame(frame()+1);
          else
          {
            currentAnimDir=-currentAnimDir;
            if (frame()>0) setFrame(frame()-1);
          }
        }
      break;
      case 3:
        if (frame()+1 <= animb[animationNumber]) setFrame(frame()+1);
        else setFrame(anima[animationNumber]);
      break;
      case -3:
        if (frame()-1 >= anima[animationNumber]) setFrame(frame()-1);
        else setFrame(animb[animationNumber]);
      break;
      default: //0
        isAnim=false;
        setFrame(0);
    }
    if (emitsignal) isAnim=false;

    animSpeedCnt=0;
  }

  // Move
  if (moveStatus)
  {
    double dx,dy;
    double vx,vy;
    dx=targetX-x();
    dy=targetY-y();
    // pure x,y movements
    vx=0.0;
    vy=0.0;
    if (fabs(dy)<=0.0001)
    {
      if (dx>0.0) vx=speed;
      else if (dx<0.0) vx=-speed;
      else vx=0.0;
    }
    else if (fabs(dx)<=0.0001)
    {
      if (dy>0.0) vy=speed;
      else if (dy<0.0) vy=-speed;
      else vy=0.0;
    }
    else // diagonal
    {
      double alpha=atan2(dy,dx);
      vx=cos(alpha)*speed;
      vy=sin(alpha)*speed;
    }

    if (fabs(dx)<=fabs(vx) && fabs(dy)<=fabs(vy))
    {
      move(targetX,targetY);
      moveStatus=0;
     // if (mEmitter) mEmitter->emitSignal(this,1);
     // else kdDebug() << "Error: emitter not defined" <<endl;
      emitsignal=1;
    }
    else if (fabs(dx)<=fabs(vx))
    {
      moveBy(dx,vy);
      isMoving=true;
    }
    else if (fabs(dy)<=fabs(vy))
    {
      moveBy(vx,dy);
      isMoving=true;
    }
    else
    {
      moveBy(vx,vy);
      isMoving=true;
    }
  }

  if (!isAnim && !isMoving)
  {
    setAnimated(false);
  }

  if (mEmitter && emitsignal)
  {
    //kdDebug() << " ADVANCE emits signal " << emitsignal << " for item "<< this << endl;
    mEmitter->emitSignal(this,emitsignal);
  }
}
void PSprite::setAnimation(int no,double animmodify)
{

  if (no<0||no>=MAX_ANIM )
  {
    #ifdef DEBUG
    cout << "ERROR: Animation number "<<no << " not supported " << endl;
    #endif
    return ;
  }
  //if (no==animationNumber) return ;
  mAnimModify=animmodify;
  animationNumber=no;
  animSpeedCnt=0;
  // char on ARM seems to be unsigned
  int direction=((int)animdir[no])-ARM_CHAR;
  currentAnimDir=direction;
  if (direction>=0)
  {
    #ifdef DEBUG
    if (animspeed[no]>10)
      cout << this <<"next srpite sets frame A to " <<(int)anima[no]<<" old "<<frame()<<endl;
    #endif
    setFrame(anima[no]);
  }
  else if (direction<0)
  {
    #ifdef DEBUG
    if (animspeed[no]>10)
      cout << this << "next srpite sets frame B to " <<(int)animb[no]<<" old "<<frame()<<endl;
    #endif
    setFrame(animb[no]);
  }
  if (direction!=0 && animb[no]>=anima[no]) setAnimated(true);
  else setAnimated(false);
  #ifdef DEBUG
  if (animspeed[no]>10)
  {
    cout << this << " setAnimation("<<no<<") delay="<<animspeed[no]*animmodify<<endl;
    cout << " frames="<<(int)anima[no]<<"->"<<(int)animb[no]<<" mode="<<(int)animdir[no]<<endl;
    cout << "    frameno=" <<frame() << " direction="<<(int)direction<<endl;
  }
  #endif
}
void PSprite::prepareAnimation(int no,char a,char b, char mode, char speed)
{
  if (no<0 || no>=MAX_ANIM)
  {
    #ifdef DEBUG
    cout << "ERROR: Animation number " << no << " not supported!"<<endl;
    #endif
    return ;
  }
  if (a>b)
  {
    #ifdef DEBUG
    cout << "ERROR: Animation start frame " << a << " bigger than end frame "<<b<<endl;
    #endif
    return ;
  }
  if (a<=0) a=0;
  if (b>=frameCount())
  {
    #ifdef DEBUG
    cout << "WARNING: prepareAnimation("<<no<<","<<a<<","<<b<<" over frameCount()"<<frameCount()<<"!!!"<<endl;
    #endif
    b=frameCount()-1;
  }
  anima[no]=a;
  animb[no]=b;
  animdir[no]=mode;
  animspeed[no]=speed;
//  kdDebug() <<" prepareAnim: a="<<a<<" b="<<b<< " dir=" <<mode<<" speed="<<speed<<endl;
}

QObject *PSprite::emitter(bool createnew)
{
  if (!mEmitter)
  {
    if (createnew)
    {
      mEmitter=new SpriteNotify;
      //kdDebug() << " new emitter" << mEmitter<< " for sprite " << this << endl;
    }
  }
  else
  {
    mEmitter->refCnt++;
    //kdDebug() << " emitter" << mEmitter << " refcnt=" << mEmitter->refCnt<< " for sprite " << this << endl;
  }
  return (QObject *)mEmitter;
}

void PSprite::deleteEmitter()
{
  //kdDebug() << " delete emitter" << mEmitter << " refcnt=" << mEmitter->refCnt<< " for sprite " << this << endl;
  if (mEmitter && mEmitter->refCnt<=1)
  {
    delete mEmitter;
    mEmitter=0;
  }
  else mEmitter->refCnt--;
}

void PSprite::moveTo(int x,int y)
{
//  kdDebug()  <<"PSprite("<<this<<")::moveTo" << x << ", " << y << endl;
  targetX=x;
  targetY=y;
  moveStatus=1;
  setAnimated(true);
}

QCanvasPixmap* PSprite::pixmap() const
{
  return image();
}

SpriteCache::SpriteCache()
{
//  kdDebug() << " creating spritecache " << this << endl;
  pixmapArray=0;
}
SpriteCache::~SpriteCache()
{
  if (pixmapArray) delete pixmapArray;
  pixmapArray=0;
//  kdDebug() << " deleting spritecache " << this << endl;
}
